==> Botan-1.1.2.tgz: Botan-1.1.2/src/get_mode.cpp <==
/*************************************************
* Cipher Mode Retrieval Source File              *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/lookup.h>
#include <botan/filters.h>
#include <botan/ecb.h>
#include <botan/cbc.h>
#include <botan/cts.h>
#include <botan/cfb.h>
#include <botan/ofb.h>
#include <botan/ctr.h>
#include <botan/util.h>

namespace Botan {

namespace {

/*************************************************
* Return the default padding method for a mode   *
*************************************************/
std::string default_padding(const std::string& mode)
   {
   if(mode == "ECB" || mode == "CFB" || mode == "OFB" || mode == "CTR-BE")
      return "NoPadding";
   else if(mode == "CBC")
      return "PKCS7";
   else
      throw Algorithm_Not_Found(mode);
   }

/*************************************************
* Simply return a mode object of choice          *
*************************************************/
Keyed_Filter* get_mode(Cipher_Dir direction, const std::string& cipher,
                       const SymmetricKey& key, const InitializationVector& iv,
                       const std::string& mode, const std::string& pad = "",
                       u32bit cfb_bits = 0)
   {
   if(mode == "ECB")
      {
      if(direction == ENCRYPTION)
         return new ECB_Encryption(cipher, key);
      else
         return new ECB_Decryption(cipher, key);
      }
   else if(mode == "CFB")
      {
      if(direction == ENCRYPTION)
         return new CFB_Encryption(cipher, key, iv, cfb_bits);
      else
         return new CFB_Decryption(cipher, key, iv, cfb_bits);
      }
   else if(mode == "CBC")
      {
      if(direction == ENCRYPTION)
         return new CBC_Encryption(cipher, pad, key, iv);
      else
         return new CBC_Decryption(cipher, pad, key, iv);
      }
   else if(mode == "CTS")
      {
      if(direction == ENCRYPTION)
         return new CTS_Encryption(cipher, key, iv);
      else
         return new CTS_Decryption(cipher, key, iv);
      }
   else
      throw Internal_Error("get_mode: " + cipher + "/" + mode + "/" + pad);
   }

}

/*************************************************
* Get a cipher object                            *
*************************************************/
Keyed_Filter* get_cipher(const std::string& algo_spec, const SymmetricKey& key,
                         const InitializationVector& iv, Cipher_Dir direction)
   {
   struct Invalid_Combination : public Exception
      {
      Invalid_Combination(const std::string& mode, const std::string& pad) :
         Exception("Invalid mode/padding pair: " + mode + "/" + pad) {}
      };

   std::vector<std::string> algo_parts = split_on_slash(algo_spec);
   if(algo_parts.size() == 0)
      throw Invalid_Algorithm_Name(algo_spec);

   const std::string cipher = algo_parts[0];

   if(have_stream_cipher(cipher))
      {
      if(algo_parts.size() != 1)
         throw Invalid_Algorithm_Name(algo_spec);
      return new StreamCipher_Filter(cipher, key, iv);
      }
   else if(have_block_cipher(cipher))
      {
      if(algo_parts.size() != 2 && algo_parts.size() != 3)
         throw Invalid_Algorithm_Name(algo_spec);
      std::string mode = algo_parts[1];
      u32bit cfb_bits = 0;
      if(mode.find("CFB") != std::string::npos)
         {
         std::vector<std::string> cfb_spec = parse_algorithm_name(mode);
         mode = cfb_spec[0];
         if(cfb_spec.size() == 1)
            cfb_bits = 8*block_size_of(cipher);
         else if(cfb_spec.size() == 2)
            cfb_bits = to_u32bit(cfb_spec[1]);
         else
            throw Invalid_Algorithm_Name(algo_spec);
         }

      std::string padding;
      if(algo_parts.size() == 3)
         padding = algo_parts[2];
      else
         padding = default_padding(mode);

      if(mode == "ECB" && padding != "NoPadding")
         throw Invalid_Combination(mode, padding);
      if(mode == "CFB" || mode == "OFB" || mode == "CTR-BE")
         if(padding != "NoPadding")
            throw Invalid_Combination(mode, padding);

      if(mode == "ECB")
         {
         if(iv.length())
            throw Invalid_Argument("ECB cannot be used with an IV");
         return get_mode(direction, cipher, key, iv, mode);
         }
      else if(mode == "CBC")
         {
         if(padding == "CTS")
            return get_mode(direction, cipher, key, iv, "CTS");
         else
            return get_mode(direction, cipher, key, iv, mode, padding);
         }
      else if(mode == "CFB")
         return get_mode(direction, cipher, key, iv, mode, "", cfb_bits);
      else if(mode == "OFB")
         return new OFB(cipher, key, iv);
      else if(mode == "CTR-BE")
         return new CTR_BE(cipher, key, iv);
      else
         throw Algorithm_Not_Found(algo_spec);
      }
   throw Algorithm_Not_Found(algo_spec);
   }

}

==> Botan-1.1.5.tgz: Botan-1.1.5/src/get_mode.cpp <==
/*************************************************
* Cipher Mode Retrieval Source File              *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/lookup.h>
#include <botan/filters.h>
#include <botan/ecb.h>
#include <botan/cbc.h>
#include <botan/cts.h>
#include <botan/cfb.h>
#include <botan/ofb.h>
#include <botan/ctr.h>
#include <botan/util.h>

namespace Botan {

namespace {

/*************************************************
* Return the default padding method for a mode   *
*************************************************/
std::string default_padding(const std::string& mode)
   {
   if(mode == "ECB" || mode == "CFB" || mode == "OFB" || mode == "CTR-BE")
      return "NoPadding";
   else if(mode == "CBC")
      return "PKCS7";
   else
      throw Algorithm_Not_Found(mode);
   }

/*************************************************
* Simply return a mode object of choice          *
*************************************************/
Keyed_Filter* get_mode(Cipher_Dir direction, const std::string& cipher,
                       const SymmetricKey& key, const InitializationVector& iv,
                       const std::string& mode, const std::string& pad = "",
                       u32bit cfb_bits = 0)
   {
   if(mode == "ECB")
      {
      if(direction == ENCRYPTION)
         return new ECB_Encryption(cipher, pad, key);
      else
         return new ECB_Decryption(cipher, pad, key);
      }
   else if(mode == "CFB")
      {
      if(direction == ENCRYPTION)
         return new CFB_Encryption(cipher, key, iv, cfb_bits);
      else
         return new CFB_Decryption(cipher, key, iv, cfb_bits);
      }
   else if(mode == "CBC")
      {
      if(direction == ENCRYPTION)
         return new CBC_Encryption(cipher, pad, key, iv);
      else
         return new CBC_Decryption(cipher, pad, key, iv);
      }
   else if(mode == "CTS")
      {
      if(direction == ENCRYPTION)
         return new CTS_Encryption(cipher, key, iv);
      else
         return new CTS_Decryption(cipher, key, iv);
      }
   else
      throw Internal_Error("get_mode: " + cipher + "/" + mode + "/" + pad);
   }

}

/*************************************************
* Get a cipher object                            *
*************************************************/
Keyed_Filter* get_cipher(const std::string& algo_spec, const SymmetricKey& key,
                         const InitializationVector& iv, Cipher_Dir direction)
   {
   struct Invalid_Combination : public Exception
      {
      Invalid_Combination(const std::string& mode, const std::string& pad) :
         Exception("Invalid mode/padding pair: " + mode + "/" + pad) {}
      };

   std::vector<std::string> algo_parts = split_on_slash(algo_spec);
   if(algo_parts.size() == 0)
      throw Invalid_Algorithm_Name(algo_spec);

   const std::string cipher = algo_parts[0];

   if(have_stream_cipher(cipher))
      {
      if(algo_parts.size() != 1)
         throw Invalid_Algorithm_Name(algo_spec);
      return new StreamCipher_Filter(cipher, key, iv);
      }
   else if(have_block_cipher(cipher))
      {
      if(algo_parts.size() != 2 && algo_parts.size() != 3)
         throw Invalid_Algorithm_Name(algo_spec);
      std::string mode = algo_parts[1];
      u32bit cfb_bits = 0;
      if(mode.find("CFB") != std::string::npos)
         {
         std::vector<std::string> cfb_spec = parse_algorithm_name(mode);
         mode = cfb_spec[0];
         if(cfb_spec.size() == 1)
            cfb_bits = 8*block_size_of(cipher);
         else if(cfb_spec.size() == 2)
            cfb_bits = to_u32bit(cfb_spec[1]);
         else
            throw Invalid_Algorithm_Name(algo_spec);
         }

      std::string padding;
      if(algo_parts.size() == 3)
         padding = algo_parts[2];
      else
         padding = default_padding(mode);

      if(mode == "ECB" && padding == "CTS")
         throw Invalid_Combination(mode, padding);
      if(mode == "CFB" || mode == "OFB" || mode == "CTR-BE")
         if(padding != "NoPadding")
            throw Invalid_Combination(mode, padding);

      if(mode == "ECB")
         {
         if(iv.length())
            throw Invalid_Argument("ECB cannot be used with an IV");
         return get_mode(direction, cipher, key, iv, mode, padding);
         }
      else if(mode == "CBC")
         {
         if(padding == "CTS")
            return get_mode(direction, cipher, key, iv, "CTS");
         else
            return get_mode(direction, cipher, key, iv, mode, padding);
         }
      else if(mode == "CFB")
         return get_mode(direction, cipher, key, iv, mode, "", cfb_bits);
      else if(mode == "OFB")
         return new OFB(cipher, key, iv);
      else if(mode == "CTR-BE")
         return new CTR_BE(cipher, key, iv);
      else
         throw Algorithm_Not_Found(algo_spec);
      }
   throw Algorithm_Not_Found(algo_spec);
   }

}
